@@ -1,7 +1,7 @@
 /*
  * Small program to output the Nth message from an mbox file on stdin.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include <stdio.h>
@@ -15,7 +15,7 @@
 int main(int argc, char **argv)
 {
 	int dispmsg, prevnl, msgnum;
-	char buf[1024];
+	char buf[1024];		/* RATS: ignore (checked) */
 
 	if (argc != 2) {
 		fprintf(stderr, "Usage: mboxsplit MESSAGENUM\n");
