@@ -1,11 +1,12 @@
 /*
  * Binary tree database backend.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
 #include "database.h"
+#include "log.h"
 
 #ifdef USING_BTREE
 
@@ -62,47 +63,6 @@ struct qdbint_s {			 /* database state */
 static char *bt_lasterror = "";
 
 
-/*
- * Block common interrupt signals, so a user doesn't accidentally kill this
- * process while it's doing something critical to data integrity.
- */
-static void dbbt_sig_block(void)
-{
-#ifdef SIGHUP
-	signal(SIGHUP, SIG_IGN);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGINT
-	signal(SIGINT, SIG_IGN);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGQUIT
-	signal(SIGQUIT, SIG_IGN);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGTERM
-	signal(SIGTERM, SIG_IGN);	    /* RATS: ignore (OK) */
-#endif
-}
-
-
-/*
- * Unblock previously blocked signals.
- */
-static void dbbt_sig_unblock(void)
-{
-#ifdef SIGHUP
-	signal(SIGHUP, SIG_DFL);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGINT
-	signal(SIGINT, SIG_DFL);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGQUIT
-	signal(SIGQUIT, SIG_DFL);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGTERM
-	signal(SIGTERM, SIG_DFL);	    /* RATS: ignore (OK) */
-#endif
-}
-
-
 #ifdef HAVE_FCNTL
 /*
  * Obtain / release a read or write lock on the database. Returns nonzero on
@@ -110,33 +70,12 @@ static void dbbt_sig_unblock(void)
  */
 static int dbbt_lock(qdbint_t db, int lock_type)
 {
-	struct flock lock;
-
-	if ((lock_type == F_UNLCK) && (db->lockcount > 1)) {
-		db->lockcount--;
-		return 0;
-	} else if ((lock_type != F_UNLCK) && (db->lockcount > 0)) {
-		db->lockcount++;
-		return 0;
-	}
-
-	lock.l_whence = SEEK_SET;
-	lock.l_start = 0;
-	lock.l_len = 0;
-
-	lock.l_type = lock_type;
-
-	if (fcntl(db->fd, F_SETLKW, &lock)) {
+	int ret;
+	ret = qdb_int__lock(db->fd, lock_type, &(db->lockcount));
+	if (ret != 0) {
 		bt_lasterror = strerror(errno);
 		return 1;
 	}
-
-	if (lock_type == F_UNLCK) {
-		db->lockcount--;
-	} else {
-		db->lockcount++;
-	}
-
 	return 0;
 }
 #endif				/* HAVE_FCNTL */
@@ -215,6 +154,10 @@ static int dbbt_chunkread(void *ptr, int size, int nmemb, qdbint_t db)
 		return 0;
 
 	available = db->size - db->filepos;
+
+	if (available <= 0)
+		return 0;
+
 	copyable = available / size;
 	if (copyable < nmemb)
 		nmemb = copyable;
@@ -252,11 +195,12 @@ static int dbbt_chunkwrite(void *ptr, int size, int nmemb, qdbint_t db)
 	if (copyable < nmemb) {
 		int retval;
 
-		lseek(db->fd, db->filepos, SEEK_SET);
-		retval = dbbt_chunkwrite_raw(ptr, size, nmemb, db->fd);
 		munmap(db->data, db->size);
 		db->data = 0;
 
+		lseek(db->fd, db->filepos, SEEK_SET);
+		retval = dbbt_chunkwrite_raw(ptr, size, nmemb, db->fd);
+
 		db->size = lseek(db->fd, 0, SEEK_END);
 		db->data =
 		    mmap(NULL, db->size, PROT_READ | PROT_WRITE,
@@ -528,7 +472,6 @@ qdbint_t qdb_btree_open(const char *file, qdb_open_t method)
 
 	db->filename = strdup(file);
 	db->fd = fd;
-	db->size = dbbt_lseek(db, 0, SEEK_END);
 	db->modified = 0;
 
 #ifdef HAVE_FCNTL
@@ -543,13 +486,15 @@ qdbint_t qdb_btree_open(const char *file, qdb_open_t method)
 	}
 #endif
 
+	db->size = dbbt_lseek(db, 0, SEEK_END);
+
 	/*
 	 * If this is a new database, and it's not readonly, write our
 	 * header to it and return.
 	 */
 	if ((db->size < 5) && (method != QDB_READONLY)) {
 		dbbt_lseek(db, 0, SEEK_SET);
-		write(db->fd, "QSF1", 4);
+		dbbt_chunkwrite_raw("QSF1", 4, 1, db->fd);
 		db->size = 4;
 		db->modified = 1;
 #ifdef HAVE_MMAP
@@ -637,16 +582,16 @@ void qdb_btree_close(qdbint_t db)
 	if (db == NULL)
 		return;
 
-#ifdef HAVE_FCNTL
-	while (db->lockcount > 0)
-		dbbt_lock(db, F_UNLCK);
-#endif
-
 #ifdef HAVE_MMAP
 	if (db->data)
 		munmap(db->data, db->size);
 #endif
 
+#ifdef HAVE_FCNTL
+	while (db->lockcount > 0)
+		dbbt_lock(db, F_UNLCK);
+#endif
+
 	close(db->fd);
 
 	if (db->modified && db->filename) {
@@ -744,17 +689,17 @@ int qdb_btree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	if (val.size > 2 * sizeof(long))
 		record.data[2] = ((long *) val.data)[2];
 
-	dbbt_sig_block();
+	qdb_int__sig_block();
 
 	/*
 	 * Record exists - overwrite it.
 	 */
 	if (offset > 0) {
 		if (dbbt_write_record(db, offset, &record)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 0;
 	}
 
@@ -769,13 +714,13 @@ int qdb_btree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 
 		if (dbbt_lseek(db, 4, SEEK_SET) == (off_t) - 1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 		offset = 4 + 2 * sizeof(offset);
 		if (dbbt_chunkwrite(&offset, sizeof(offset), 1, db) < 1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 
@@ -784,25 +729,25 @@ int qdb_btree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 		offset = 0;
 		if (dbbt_chunkwrite(&offset, sizeof(offset), 1, db) < 1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 
 		head.lower = (4 + 2 * sizeof(offset)) + BT_RECORD_SIZE;
 		if (dbbt_chunkwrite(&head, BT_RECORD_SIZE, 1, db) < 1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 
 		if (dbbt_chunkwrite(&record, BT_RECORD_SIZE, 1, db) < 1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 
 		db->size = dbbt_lseek(db, 0, SEEK_CUR);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 0;
 	}
 
@@ -811,12 +756,12 @@ int qdb_btree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	 */
 	if (dbbt_lseek(db, 4 + sizeof(offset), SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 	if (dbbt_chunkread(&offset, sizeof(offset), 1, db) < 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
@@ -830,7 +775,7 @@ int qdb_btree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	 */
 	if (dbbt_lseek(db, offset, SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 	if ((offset < 5)
@@ -841,17 +786,17 @@ int qdb_btree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	}
 	if (dbbt_lseek(db, 4 + sizeof(offset), SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 	if (dbbt_chunkwrite(&nextfree, sizeof(nextfree), 1, db) < 0) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
 	if (dbbt_write_record(db, offset, &record)) {
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
@@ -860,7 +805,7 @@ int qdb_btree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	 */
 	if (parent > 0) {
 		if (dbbt_read_record(db, parent, &record)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 		if (x < 0) {
@@ -869,12 +814,12 @@ int qdb_btree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 			record.higher = offset;
 		}
 		if (dbbt_write_record(db, parent, &record)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 	}
 
-	dbbt_sig_unblock();
+	qdb_int__sig_unblock();
 
 	return 0;
 }
@@ -980,13 +925,13 @@ static int qdb_btree_delete__rewrite(qdbint_t db, bt_record_t record)
 	newrec.lower = record->lower;
 	newrec.higher = record->higher;
 
-	dbbt_sig_block();
+	qdb_int__sig_block();
 	if (dbbt_write_record(db, offset, &newrec)) {
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
-	dbbt_sig_unblock();
+	qdb_int__sig_unblock();
 
 	return 0;
 }
@@ -1025,14 +970,14 @@ int qdb_btree_delete(qdbint_t db, qdb_datum key)
 			return 1;
 	}
 
-	dbbt_sig_block();
+	qdb_int__sig_block();
 
 	/*
 	 * Remove the link to this record from its parent.
 	 */
 	if (parent > 0) {
 		if (dbbt_read_record(db, parent, &recparent)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 		if (x < 0) {
@@ -1041,7 +986,7 @@ int qdb_btree_delete(qdbint_t db, qdb_datum key)
 			recparent.higher = 0;
 		}
 		if (dbbt_write_record(db, parent, &recparent)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 	}
@@ -1051,7 +996,7 @@ int qdb_btree_delete(qdbint_t db, qdb_datum key)
 	 * linked list.
 	 */
 	if (dbbt_markfree(db, offset)) {
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
@@ -1060,18 +1005,18 @@ int qdb_btree_delete(qdbint_t db, qdb_datum key)
 	 */
 	if (record.lower > 0) {
 		if (dbbt_markfree(db, record.lower)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 	}
 	if (record.higher > 0) {
 		if (dbbt_markfree(db, record.higher)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 	}
 
-	dbbt_sig_unblock();
+	qdb_int__sig_unblock();
 
 	/*
 	 * Re-attach any children of this record to the database.
@@ -1129,11 +1074,20 @@ void qdb_btree_optimise(qdbint_t db)
 	int newdbfd, got;
 
 #ifdef P_tmpdir
-	sprintf(filename, "%.*s", (int) (sizeof(filename) - 1),
+#ifdef HAVE_SNPRINTF
+	snprintf(filename, sizeof(filename),
+#else
+	sprintf(filename,		    /* RATS: ignore (checked) */
+#endif
+		"%.*s", (int) (sizeof(filename) - 1),
 		P_tmpdir "/qsfXXXXXX");
 #else
-	sprintf(filename, "%.*s", (int) (sizeof(filename) - 1),
-		"/tmp/qsfXXXXXX");
+#ifdef HAVE_SNPRINTF
+	snprintf(filename, sizeof(filename),
+#else
+	sprintf(filename,		    /* RATS: ignore (checked) */
+#endif
+		"%.*s", (int) (sizeof(filename) - 1), "/tmp/qsfXXXXXX");
 #endif
 
 #ifdef HAVE_MKSTEMP
@@ -1180,7 +1134,7 @@ void qdb_btree_optimise(qdbint_t db)
 	db->data = 0;
 #endif
 
-	dbbt_sig_block();
+	qdb_int__sig_block();
 
 	dbbt_lseek(db, 0, SEEK_SET);
 	lseek(newdbfd, 0, SEEK_SET);
@@ -1194,12 +1148,15 @@ void qdb_btree_optimise(qdbint_t db)
 
 	} while (got > 0);
 
-	ftruncate(db->fd, lseek(db->fd, 0, SEEK_CUR));
+	if (ftruncate(db->fd, lseek(db->fd, 0, SEEK_CUR)) != 0) {
+		log_add(0, "%s: failed to truncate database: %s",
+			db->filename, strerror(errno));
+	}
 
 	close(newdbfd);
 	remove(filename);
 
-	dbbt_sig_unblock();
+	qdb_int__sig_unblock();
 
 	db->size = dbbt_lseek(db, 0, SEEK_END);
 	db->gotheadoffs = 0;
