@@ -1,7 +1,7 @@
 /*
  * Make our db functions a wrapper for GDBM.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -99,28 +99,43 @@ qdbint_t qdb_gdbm_open(const char *file, qdb_open_t method)
 {
 	GDBM_FILE dbf = NULL;
 	qdbint_t db;
+	int tries;
 
 	if (strncasecmp(file, "gdbm:", 5) == 0)
 		file += 5;
 
-	switch (method) {
-	case QDB_NEW:
-		dbf =
-		    gdbm_open((char *) file, 512, GDBM_NEWDB,
-			      S_IRUSR | S_IWUSR, NULL);
-		break;
-	case QDB_READONLY:
-		dbf =
-		    gdbm_open((char *) file, 512, GDBM_READER, S_IRUSR,
-			      NULL);
-		break;
-	case QDB_READWRITE:
-		dbf =
-		    gdbm_open((char *) file, 512, GDBM_WRCREAT,
-			      S_IRUSR | S_IWUSR, NULL);
-		break;
-	default:
-		break;
+	for (tries = 0; tries < 60 && dbf == NULL; tries++) {
+		switch (method) {
+		case QDB_NEW:
+			dbf =
+			    gdbm_open((char *) file, 512, GDBM_NEWDB,
+				      S_IRUSR | S_IWUSR, NULL);
+			break;
+		case QDB_READONLY:
+			dbf =
+			    gdbm_open((char *) file, 512, GDBM_READER,
+				      S_IRUSR, NULL);
+			break;
+		case QDB_READWRITE:
+			dbf =
+			    gdbm_open((char *) file, 512, GDBM_WRCREAT,
+				      S_IRUSR | S_IWUSR, NULL);
+			break;
+		default:
+			break;
+		}
+
+		if (dbf != NULL)
+			break;
+
+		switch (gdbm_errno) {
+		case GDBM_CANT_BE_READER:
+		case GDBM_CANT_BE_WRITER:
+			sleep(1);
+			break;
+		default:
+			return NULL;
+		}
 	}
 
 	if (dbf == NULL)
@@ -290,23 +305,6 @@ char *qdb_gdbm_error(void)
 }
 
 
-/*
- * Temporarily release the lock on the database.
- */
-void qdb_gdbm_unlock(qdbint_t db)
-{
-	/* no-op */
-}
-
-
-/*
- * Reassert the lock on the database.
- */
-void qdb_gdbm_relock(qdbint_t db)
-{
-	/* no-op */
-}
-
 #endif				/* USING_GDBM */
 
 /* EOF */
