@@ -1,7 +1,7 @@
 /*
  * Main entry point for database functions.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -9,6 +9,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
+#ifdef HAVE_FCNTL
+#include <fcntl.h>
+#endif
 
 #ifdef USING_OBTREE
 int qdb_obtree_identify(const char *);
@@ -20,7 +24,6 @@ int qdb_obtree_store(qdbint_t, qdb_datum, qdb_datum);
 int qdb_obtree_delete(qdbint_t, qdb_datum);
 qdb_datum qdb_obtree_firstkey(qdbint_t);
 qdb_datum qdb_obtree_nextkey(qdbint_t, qdb_datum);
-void qdb_obtree_optimise(qdbint_t);
 char *qdb_obtree_error(void);
 void qdb_obtree_unlock(qdbint_t);
 void qdb_obtree_relock(qdbint_t);
@@ -40,6 +43,20 @@ char *qdb_btree_error(void);
 void qdb_btree_unlock(qdbint_t);
 void qdb_btree_relock(qdbint_t);
 #endif				/* USING_BTREE */
+#ifdef USING_LIST
+int qdb_list_identify(const char *);
+qdbint_t qdb_list_open(const char *, qdb_open_t);
+int qdb_list_fd(qdbint_t);
+void qdb_list_close(qdbint_t);
+qdb_datum qdb_list_fetch(qdbint_t, qdb_datum);
+int qdb_list_store(qdbint_t, qdb_datum, qdb_datum);
+int qdb_list_delete(qdbint_t, qdb_datum);
+qdb_datum qdb_list_firstkey(qdbint_t);
+qdb_datum qdb_list_nextkey(qdbint_t, qdb_datum);
+char *qdb_list_error(void);
+void qdb_list_restore_start(qdbint_t);
+void qdb_list_restore_end(qdbint_t);
+#endif				/* USING_LIST */
 #ifdef USING_GDBM
 int qdb_gdbm_identify(const char *);
 qdbint_t qdb_gdbm_open(const char *, qdb_open_t);
@@ -52,8 +69,6 @@ qdb_datum qdb_gdbm_firstkey(qdbint_t);
 qdb_datum qdb_gdbm_nextkey(qdbint_t, qdb_datum);
 void qdb_gdbm_optimise(qdbint_t);
 char *qdb_gdbm_error(void);
-void qdb_gdbm_unlock(qdbint_t);
-void qdb_gdbm_relock(qdbint_t);
 #endif				/* USING_GDBM */
 #ifdef USING_MYSQL
 int qdb_mysql_identify(const char *);
@@ -65,10 +80,7 @@ int qdb_mysql_store(qdbint_t, qdb_datum, qdb_datum);
 int qdb_mysql_delete(qdbint_t, qdb_datum);
 qdb_datum qdb_mysql_firstkey(qdbint_t);
 qdb_datum qdb_mysql_nextkey(qdbint_t, qdb_datum);
-void qdb_mysql_optimise(qdbint_t);
 char *qdb_mysql_error(void);
-void qdb_mysql_unlock(qdbint_t);
-void qdb_mysql_relock(qdbint_t);
 #endif				/* USING_MYSQL */
 #ifdef USING_SQLITE
 int qdb_sqlite_identify(const char *);
@@ -80,7 +92,6 @@ int qdb_sqlite_store(qdbint_t, qdb_datum, qdb_datum);
 int qdb_sqlite_delete(qdbint_t, qdb_datum);
 qdb_datum qdb_sqlite_firstkey(qdbint_t);
 qdb_datum qdb_sqlite_nextkey(qdbint_t, qdb_datum);
-void qdb_sqlite_optimise(qdbint_t);
 char *qdb_sqlite_error(void);
 void qdb_sqlite_unlock(qdbint_t);
 void qdb_sqlite_relock(qdbint_t);
@@ -97,10 +108,12 @@ struct qdbtype_s {
 	int (*_delete) (qdbint_t, qdb_datum);
 	 qdb_datum(*_firstkey) (qdbint_t);
 	 qdb_datum(*_nextkey) (qdbint_t, qdb_datum);
-	void (*_optimise) (qdbint_t);
 	char *(*_error) (void);
-	void (*_unlock) (qdbint_t);
-	void (*_relock) (qdbint_t);
+	void (*_optimise) (qdbint_t);	 /* optional */
+	void (*_unlock) (qdbint_t);	 /* optional */
+	void (*_relock) (qdbint_t);	 /* optional */
+	void (*_restore_start) (qdbint_t);	/* optional */
+	void (*_restore_end) (qdbint_t); /* optional */
 };
 
 struct qdb_s {
@@ -110,22 +123,24 @@ struct qdb_s {
 };
 
 static struct qdbtype_s qdb__backends[] = {
-#ifdef USING_OBTREE
+#ifdef USING_LIST
 	{
-	 "obtree",
-	 qdb_obtree_identify,
-	 qdb_obtree_open,
-	 qdb_obtree_fd,
-	 qdb_obtree_close,
-	 qdb_obtree_fetch,
-	 qdb_obtree_store,
-	 qdb_obtree_delete,
-	 qdb_obtree_firstkey,
-	 qdb_obtree_nextkey,
-	 qdb_obtree_optimise,
-	 qdb_obtree_error,
-	 qdb_obtree_unlock,
-	 qdb_obtree_relock},
+	 "list",
+	 qdb_list_identify,
+	 qdb_list_open,
+	 qdb_list_fd,
+	 qdb_list_close,
+	 qdb_list_fetch,
+	 qdb_list_store,
+	 qdb_list_delete,
+	 qdb_list_firstkey,
+	 qdb_list_nextkey,
+	 qdb_list_error,
+	 NULL,				    /* optimise */
+	 NULL,				    /* unlock */
+	 NULL,				    /* relock */
+	 qdb_list_restore_start,
+	 qdb_list_restore_end},
 #endif
 #ifdef USING_BTREE
 	{
@@ -139,10 +154,31 @@ static struct qdbtype_s qdb__backends[] = {
 	 qdb_btree_delete,
 	 qdb_btree_firstkey,
 	 qdb_btree_nextkey,
-	 qdb_btree_optimise,
 	 qdb_btree_error,
+	 qdb_btree_optimise,
 	 qdb_btree_unlock,
-	 qdb_btree_relock},
+	 qdb_btree_relock,
+	 NULL,				    /* restore_start */
+	 NULL},				    /* restore_end */
+#endif
+#ifdef USING_OBTREE
+	{
+	 "obtree",
+	 qdb_obtree_identify,
+	 qdb_obtree_open,
+	 qdb_obtree_fd,
+	 qdb_obtree_close,
+	 qdb_obtree_fetch,
+	 qdb_obtree_store,
+	 qdb_obtree_delete,
+	 qdb_obtree_firstkey,
+	 qdb_obtree_nextkey,
+	 qdb_obtree_error,
+	 NULL,				    /* optimise */
+	 qdb_obtree_unlock,
+	 qdb_obtree_relock,
+	 NULL,				    /* restore_start */
+	 NULL},				    /* restore_end */
 #endif
 #ifdef USING_GDBM
 	{
@@ -156,10 +192,12 @@ static struct qdbtype_s qdb__backends[] = {
 	 qdb_gdbm_delete,
 	 qdb_gdbm_firstkey,
 	 qdb_gdbm_nextkey,
-	 qdb_gdbm_optimise,
 	 qdb_gdbm_error,
-	 qdb_gdbm_unlock,
-	 qdb_gdbm_relock},
+	 qdb_gdbm_optimise,
+	 NULL,				    /* unlock */
+	 NULL,				    /* relock */
+	 NULL,				    /* restore_start */
+	 NULL},				    /* restore_end */
 #endif
 #ifdef USING_MYSQL
 	{
@@ -173,10 +211,12 @@ static struct qdbtype_s qdb__backends[] = {
 	 qdb_mysql_delete,
 	 qdb_mysql_firstkey,
 	 qdb_mysql_nextkey,
-	 qdb_mysql_optimise,
 	 qdb_mysql_error,
-	 qdb_mysql_unlock,
-	 qdb_mysql_relock},
+	 NULL,				    /* optimise */
+	 NULL,				    /* unlock */
+	 NULL,				    /* relock */
+	 NULL,				    /* restore_start */
+	 NULL},				    /* restore_end */
 #endif
 #ifdef USING_SQLITE
 	{
@@ -190,12 +230,14 @@ static struct qdbtype_s qdb__backends[] = {
 	 qdb_sqlite_delete,
 	 qdb_sqlite_firstkey,
 	 qdb_sqlite_nextkey,
-	 qdb_sqlite_optimise,
 	 qdb_sqlite_error,
+	 NULL,				    /* optimise */
 	 qdb_sqlite_unlock,
-	 qdb_sqlite_relock},
+	 qdb_sqlite_relock,
+	 NULL,				    /* restore_start */
+	 NULL},				    /* restore_end */
 #endif
-	{NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
+	{NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
 };
 
 static int qdb__lastbackend = 0;
@@ -242,7 +284,7 @@ qdb_t qdb_open(const char *file, qdb_open_t method)
 
 	/*
 	 * If no valid type was determined, fall back to obtree if the file
-	 * already exists, btree if not (providing they are both compiled
+	 * already exists, list if not (providing they are both compiled
 	 * in). If neither is compiled in, fall back to the first available
 	 * backend in the list.
 	 */
@@ -250,8 +292,8 @@ qdb_t qdb_open(const char *file, qdb_open_t method)
 		char *usetype = NULL;
 		FILE *fptr;
 
-#ifdef USING_BTREE
-		usetype = "btree";
+#ifdef USING_LIST
+		usetype = "list";
 #endif
 
 #ifdef USING_OBTREE
@@ -419,6 +461,8 @@ qdb_datum qdb_nextkey(qdb_t db, qdb_datum key)
 void qdb_optimise(qdb_t db)
 {
 	qdb__lastbackend = db->typeindex;
+	if (db->type->_optimise == NULL)
+		return;
 	db->type->_optimise(db->data);
 }
 
@@ -438,6 +482,8 @@ char *qdb_error(void)
 void qdb_unlock(qdb_t db)
 {
 	qdb__lastbackend = db->typeindex;
+	if (db->type->_unlock == NULL)
+		return;
 	db->type->_unlock(db->data);
 }
 
@@ -448,7 +494,124 @@ void qdb_unlock(qdb_t db)
 void qdb_relock(qdb_t db)
 {
 	qdb__lastbackend = db->typeindex;
+	if (db->type->_relock == NULL)
+		return;
 	db->type->_relock(db->data);
 }
 
+
+/*
+ * Tell the database that a restore operation is starting.
+ */
+void qdb_restore_start(qdb_t db)
+{
+	qdb__lastbackend = db->typeindex;
+	if (db->type->_restore_start == NULL)
+		return;
+	db->type->_restore_start(db->data);
+}
+
+
+/*
+ * Tell the database that a restore operation is ending.
+ */
+void qdb_restore_end(qdb_t db)
+{
+	qdb__lastbackend = db->typeindex;
+	if (db->type->_restore_end == NULL)
+		return;
+	db->type->_restore_end(db->data);
+}
+
+
+/*
+ * Utility functions that can be used by the back-end follow.
+ */
+
+
+/*
+ * Block common interrupt signals, so a user doesn't accidentally kill this
+ * process while it's doing something critical to data integrity.
+ */
+void qdb_int__sig_block(void)
+{
+#ifdef SIGHUP
+	signal(SIGHUP, SIG_IGN);	    /* RATS: ignore (OK) */
+#endif
+#ifdef SIGINT
+	signal(SIGINT, SIG_IGN);	    /* RATS: ignore (OK) */
+#endif
+#ifdef SIGQUIT
+	signal(SIGQUIT, SIG_IGN);	    /* RATS: ignore (OK) */
+#endif
+#ifdef SIGTERM
+	signal(SIGTERM, SIG_IGN);	    /* RATS: ignore (OK) */
+#endif
+}
+
+
+/*
+ * Unblock previously blocked signals.
+ */
+void qdb_int__sig_unblock(void)
+{
+#ifdef SIGHUP
+	signal(SIGHUP, SIG_DFL);	    /* RATS: ignore (OK) */
+#endif
+#ifdef SIGINT
+	signal(SIGINT, SIG_DFL);	    /* RATS: ignore (OK) */
+#endif
+#ifdef SIGQUIT
+	signal(SIGQUIT, SIG_DFL);	    /* RATS: ignore (OK) */
+#endif
+#ifdef SIGTERM
+	signal(SIGTERM, SIG_DFL);	    /* RATS: ignore (OK) */
+#endif
+}
+
+
+/*
+ * Obtain / release a read or write lock on the database. Returns zero on
+ * success, 1 on error. Blocks until a lock can be obtained. Keeps track of
+ * the number of times locked, so we don't accidentally try to lock a file
+ * descriptor twice.
+ */
+int qdb_int__lock(int fd, int lock_type, int *lockcount)
+{
+#ifdef HAVE_FCNTL
+	struct flock lock;
+	int lockdummy = 0;
+
+	if (lockcount == NULL)
+		lockcount = &lockdummy;
+
+	if ((lock_type == F_UNLCK) && (*lockcount > 1)) {
+		*lockcount = (*lockcount) - 1;
+		return 0;
+	} else if ((lock_type != F_UNLCK) && (*lockcount > 0)) {
+		*lockcount = (*lockcount) + 1;
+		return 0;
+	}
+
+	lock.l_whence = SEEK_SET;
+	lock.l_start = 0;
+	lock.l_len = 0;
+
+	lock.l_type = lock_type;
+
+	if (fcntl(fd, F_SETLKW, &lock)) {
+		return 1;
+	}
+
+	if (lock_type == F_UNLCK) {
+		*lockcount = (*lockcount) - 1;
+	} else {
+		*lockcount = (*lockcount) + 1;
+	}
+
+#endif				/* HAVE_FCNTL */
+	return 0;
+}
+
+
 /* EOF */
