@@ -1,7 +1,7 @@
 /*
  * Make our db functions a wrapper for MySQL.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -56,7 +56,15 @@ int qdb_mysql_fd(qdbint_t db)
 int qdb_mysql_identify(const char *file)
 {
 	FILE *fptr;
-	char string[4096];		 /* RATS: ignore (checked all) */
+	char string[4096] = { 0, };	    /* RATS: ignore (checked all) */
+	char host[256];			 /* RATS: ignore (checked all) */
+	char user[64];			 /* RATS: ignore (checked all) */
+	char pass[64];			 /* RATS: ignore (checked all) */
+	char database[64];		 /* RATS: ignore (checked all) */
+	char table[64];			 /* RATS: ignore (checked all) */
+	char key1[256];			 /* RATS: ignore (checked all) */
+	char key2[256];			 /* RATS: ignore (checked all) */
+	unsigned int port;
 
 	if (file == NULL)
 		return 0;
@@ -67,19 +75,36 @@ int qdb_mysql_identify(const char *file)
 	if (!fptr) {
 		sprintf(string, "%.*s", (int) (sizeof(string) - 1), file);
 	} else {
-		fread(string, 1, sizeof(string) - 1, fptr);
+		if (fread(string, 1, sizeof(string) - 1, fptr) < 1) {
+			log_add(0, "%s: read failed: %s", file,
+				strerror(errno));
+		}
 		fclose(fptr);
 	}
 
 	string[sizeof(string) - 1] = 0;
 
-	if (sscanf(string, "database=%*63[^;];"
-		   "host=%*255[^;];"
-		   "port=%*u;"
-		   "user=%*63[^;];"
-		   "pass=%*63[^;];"
-		   "table=%*63[^;];"
-		   "key1=%*255[^;];" "key2=%*255[^;]") == 8)
+	if (sscanf(string, "database=%63[^;];"
+		   "host=%255[^;];"
+		   "port=%u;"
+		   "user=%63[^;];"
+		   "pass=%63[^;];"
+		   "table=%63[^;];"
+		   "key1=%255[^;];"
+		   "key2=%255[^;]",
+		   database,
+		   host, &port, user, pass, table, key1, key2) == 8)
+		return 1;
+
+	if (sscanf(string, "database=%63[^;];"
+		   "host=%255[^;];"
+		   "port=%u;"
+		   "user=%63[^;];"
+		   "pass=;"
+		   "table=%63[^;];"
+		   "key1=%255[^;];"
+		   "key2=%255[^;]",
+		   database, host, &port, user, table, key1, key2) == 7)
 		return 1;
 
 	return 0;
@@ -182,7 +207,7 @@ qdbint_t qdb_mysql_open(const char *file, qdb_open_t method)
 	char table[64];			 /* RATS: ignore (checked all) */
 	char key1[256];			 /* RATS: ignore (checked all) */
 	char key2[256];			 /* RATS: ignore (checked all) */
-	char string[4096];		 /* RATS: ignore (checked all) */
+	char string[4096] = { 0, };	    /* RATS: ignore (checked all) */
 	unsigned int port;
 	FILE *fptr;
 	MYSQL *mdb;
@@ -203,27 +228,42 @@ qdbint_t qdb_mysql_open(const char *file, qdb_open_t method)
 	if (!fptr) {
 		sprintf(string, "%.*s", (int) (sizeof(string) - 1), file);
 	} else {
-		fread(string, 1, sizeof(string) - 1, fptr);
+		if (fread(string, 1, sizeof(string) - 1, fptr) < 1) {
+			log_add(0, "%s: read failed: %s", file,
+				strerror(errno));
+		}
 		fclose(fptr);
 	}
 
 	string[sizeof(string) - 1] = 0;
 
+	pass[0] = 0;
 	if (sscanf(string, "database=%63[^;];"
 		   "host=%255[^;];"
 		   "port=%u;"
 		   "user=%63[^;];"
-		   "pass=%63[^;];"
+		   "pass=;"
 		   "table=%63[^;];"
 		   "key1=%255[^;];"
 		   "key2=%255[^;]",
-		   database,
-		   host, &port, user, pass, table, key1, key2) < 8) {
-		sprintf(qdb_mysql__errstr, "%.*s: %.100s",
-			(int) (sizeof(qdb_mysql__errstr) - 110),
-			_("invalid DB spec"), string);
-		mysql_close(mdb);
-		return NULL;
+		   database, host, &port, user, table, key1, key2) < 7) {
+		if (sscanf(string, "database=%63[^;];"
+			   "host=%255[^;];"
+			   "port=%u;"
+			   "user=%63[^;];"
+			   "pass=%63[^;];"
+			   "table=%63[^;];"
+			   "key1=%255[^;];"
+			   "key2=%255[^;]",
+			   database,
+			   host, &port, user, pass, table, key1,
+			   key2) < 8) {
+			sprintf(qdb_mysql__errstr, "%.*s: %.100s",
+				(int) (sizeof(qdb_mysql__errstr) - 110),
+				_("invalid DB spec"), string);
+			mysql_close(mdb);
+			return NULL;
+		}
 	}
 
 	/*
@@ -325,6 +365,20 @@ qdbint_t qdb_mysql_open(const char *file, qdb_open_t method)
 			mysql_free_result(results);
 	}
 
+	switch (method) {
+	case QDB_NEW:
+		/* fall-through */
+	case QDB_READWRITE:
+		break;
+	case QDB_READONLY:
+		break;
+	default:
+		break;
+	}
+
+#ifdef HAVE_MYSQL_AUTOCOMMIT
+	mysql_autocommit(db->db, 0);
+#endif
 
 	db->results = NULL;
 
@@ -339,6 +393,9 @@ void qdb_mysql_close(qdbint_t db)
 {
 	if (db == NULL)
 		return;
+#ifdef HAVE_MYSQL_AUTOCOMMIT
+	mysql_commit(db->db);
+#endif
 	if (db->fd >= 0)
 		close(db->fd);
 	mysql_close(db->db);
@@ -478,7 +535,7 @@ int qdb_mysql_store(qdbint_t db, qdb_datum key, qdb_datum val)
 
 	data = (long *) (val.data);
 
-#if HAVE_SNPRINTF
+#ifdef HAVE_SNPRINTF
 	snprintf(val1, sizeof(val1) - 1,    /* RATS: ignore (OK) */
 		 "%ld", data[0]);
 	snprintf(val2, sizeof(val2) - 1,    /* RATS: ignore (OK) */
@@ -654,15 +711,6 @@ qdb_datum qdb_mysql_firstkey(qdbint_t db)
 }
 
 
-/*
- * Reorganise the database for better efficiency.
- */
-void qdb_mysql_optimise(qdbint_t db)
-{
-	/* (Does nothing) */
-}
-
-
 /*
  * Return a string describing the last database error to occur.
  */
@@ -672,23 +720,6 @@ char *qdb_mysql_error(void)
 }
 
 
-/*
- * Temporarily release the lock on the database.
- */
-void qdb_mysql_unlock(qdbint_t db)
-{
-	/* (Does nothing) */
-}
-
-
-/*
- * Reassert the lock on the database.
- */
-void qdb_mysql_relock(qdbint_t db)
-{
-	/* (Does nothing) */
-}
-
 #endif				/* USING_MYSQL */
 
 /* EOF */
