@@ -1,7 +1,7 @@
 /*
  * Old binary tree database backend.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -50,47 +50,6 @@ struct qdbint_s {			 /* database state */
 static char *bt_lasterror = "";
 
 
-/*
- * Block common interrupt signals, so a user doesn't accidentally kill this
- * process while it's doing something critical to data integrity.
- */
-static void dbbt_sig_block(void)
-{
-#ifdef SIGHUP
-	signal(SIGHUP, SIG_IGN);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGINT
-	signal(SIGINT, SIG_IGN);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGQUIT
-	signal(SIGQUIT, SIG_IGN);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGTERM
-	signal(SIGTERM, SIG_IGN);	    /* RATS: ignore (OK) */
-#endif
-}
-
-
-/*
- * Unblock previously blocked signals.
- */
-static void dbbt_sig_unblock(void)
-{
-#ifdef SIGHUP
-	signal(SIGHUP, SIG_DFL);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGINT
-	signal(SIGINT, SIG_DFL);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGQUIT
-	signal(SIGQUIT, SIG_DFL);	    /* RATS: ignore (OK) */
-#endif
-#ifdef SIGTERM
-	signal(SIGTERM, SIG_DFL);	    /* RATS: ignore (OK) */
-#endif
-}
-
-
 #ifdef HAVE_FCNTL
 /*
  * Obtain / release a read or write lock on the database. Returns nonzero on
@@ -98,33 +57,12 @@ static void dbbt_sig_unblock(void)
  */
 static int dbbt_lock(qdbint_t db, int lock_type)
 {
-	struct flock lock;
-
-	if ((lock_type == F_UNLCK) && (db->lockcount > 1)) {
-		db->lockcount--;
-		return 0;
-	} else if ((lock_type != F_UNLCK) && (db->lockcount > 0)) {
-		db->lockcount++;
-		return 0;
-	}
-
-	lock.l_whence = SEEK_SET;
-	lock.l_start = 0;
-	lock.l_len = 0;
-
-	lock.l_type = lock_type;
-
-	if (fcntl(db->fd, F_SETLKW, &lock)) {
+	int ret;
+	ret = qdb_int__lock(db->fd, lock_type, &(db->lockcount));
+	if (ret != 0) {
 		bt_lasterror = strerror(errno);
 		return 1;
 	}
-
-	if (lock_type == F_UNLCK) {
-		db->lockcount--;
-	} else {
-		db->lockcount++;
-	}
-
 	return 0;
 }
 #endif				/* HAVE_FCNTL */
@@ -535,17 +473,17 @@ int qdb_obtree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	record.data[0] = ((long *) val.data)[0];
 	record.data[1] = ((long *) val.data)[1];
 
-	dbbt_sig_block();
+	qdb_int__sig_block();
 
 	/*
 	 * Record exists - overwrite it.
 	 */
 	if (offset > 0) {
 		if (dbbt_write_record(db, offset, &record)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 0;
 	}
 
@@ -560,14 +498,14 @@ int qdb_obtree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 
 		if (lseek(db->fd, 0, SEEK_SET) == (off_t) - 1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 		offset = 2 * sizeof(offset);
 		if (dbbt_chunkwrite(&offset, sizeof(offset), 1, db->fd) <
 		    1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 
@@ -577,26 +515,26 @@ int qdb_obtree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 		if (dbbt_chunkwrite(&offset, sizeof(offset), 1, db->fd) <
 		    1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 
 		head.lower = (2 * sizeof(offset)) + BT_RECORD_SIZE;
 		if (dbbt_chunkwrite(&head, BT_RECORD_SIZE, 1, db->fd) < 1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 
 		if (dbbt_chunkwrite(&record, BT_RECORD_SIZE, 1, db->fd) <
 		    1) {
 			bt_lasterror = strerror(errno);
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 
 		db->size = lseek(db->fd, 0, SEEK_CUR);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 0;
 	}
 
@@ -605,12 +543,12 @@ int qdb_obtree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	 */
 	if (lseek(db->fd, sizeof(offset), SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 	if (dbbt_chunkread(&offset, sizeof(offset), 1, db->fd) < 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
@@ -624,7 +562,7 @@ int qdb_obtree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	 */
 	if (lseek(db->fd, offset, SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 	if ((offset == 0)
@@ -635,17 +573,17 @@ int qdb_obtree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	}
 	if (lseek(db->fd, sizeof(offset), SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 	if (dbbt_chunkwrite(&nextfree, sizeof(nextfree), 1, db->fd) < 0) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
 	if (dbbt_write_record(db, offset, &record)) {
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
@@ -654,7 +592,7 @@ int qdb_obtree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 	 */
 	if (parent > 0) {
 		if (dbbt_read_record(db, parent, &record)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 		if (x < 0) {
@@ -663,12 +601,12 @@ int qdb_obtree_store(qdbint_t db, qdb_datum key, qdb_datum val)
 			record.higher = offset;
 		}
 		if (dbbt_write_record(db, parent, &record)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 	}
 
-	dbbt_sig_unblock();
+	qdb_int__sig_unblock();
 
 	return 0;
 }
@@ -815,14 +753,14 @@ int qdb_obtree_delete(qdbint_t db, qdb_datum key)
 			return 1;
 	}
 
-	dbbt_sig_block();
+	qdb_int__sig_block();
 
 	/*
 	 * Remove the link to this record from its parent.
 	 */
 	if (parent > 0) {
 		if (dbbt_read_record(db, parent, &recparent)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 		if (x < 0) {
@@ -831,7 +769,7 @@ int qdb_obtree_delete(qdbint_t db, qdb_datum key)
 			recparent.higher = 0;
 		}
 		if (dbbt_write_record(db, parent, &recparent)) {
-			dbbt_sig_unblock();
+			qdb_int__sig_unblock();
 			return 1;
 		}
 	}
@@ -853,27 +791,27 @@ int qdb_obtree_delete(qdbint_t db, qdb_datum key)
 	 */
 	if (lseek(db->fd, sizeof(offset), SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
 	nextfree = 0x80000000;
 	if (dbbt_chunkread(&nextfree, sizeof(nextfree), 1, db->fd) < 0) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
 	if (lseek(db->fd, sizeof(offset), SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
 	offset |= 0x80000000;
 	if (dbbt_chunkwrite(&offset, sizeof(offset), 1, db->fd) < 0) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
@@ -881,17 +819,17 @@ int qdb_obtree_delete(qdbint_t db, qdb_datum key)
 
 	if (lseek(db->fd, offset, SEEK_SET) == (off_t) - 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
 	if (dbbt_chunkwrite(&nextfree, sizeof(nextfree), 1, db->fd) < 1) {
 		bt_lasterror = strerror(errno);
-		dbbt_sig_unblock();
+		qdb_int__sig_unblock();
 		return 1;
 	}
 
-	dbbt_sig_unblock();
+	qdb_int__sig_unblock();
 
 	return 0;
 }
@@ -920,15 +858,6 @@ void qdb_obtree_relock(qdbint_t db)
 }
 
 
-/*
- * Reorganise the database for better efficiency.
- */
-void qdb_obtree_optimise(qdbint_t db)
-{
-	/* No-op. */
-}
-
-
 /*
  * Return a string describing the last database error to occur.
  */
