@@ -1,7 +1,7 @@
 /*
  * Make our db functions a wrapper for SQLite.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -93,6 +93,8 @@ void qdb_sqlite_unlock(qdbint_t db)
  */
 void qdb_sqlite_relock(qdbint_t db)
 {
+	int tries, ret;
+
 	if (db == NULL)
 		return;
 
@@ -103,13 +105,25 @@ void qdb_sqlite_relock(qdbint_t db)
 	 * We "lock" by starting a new transaction.
 	 */
 
-	sqlite_exec(db->db, "BEGIN", NULL, NULL, &qdb_sqlite__errptr);
-	if (qdb_sqlite__errptr) {
-		sqlite_freemem(qdb_sqlite__errptr);
-		qdb_sqlite__errptr = 0;
+	for (tries = 0; tries < 60 && !db->locked; tries++) {
+		ret =
+		    sqlite_exec(db->db, "BEGIN", NULL, NULL,
+				&qdb_sqlite__errptr);
+		if (qdb_sqlite__errptr) {
+			sqlite_freemem(qdb_sqlite__errptr);
+			qdb_sqlite__errptr = 0;
+		}
+		switch (ret) {
+		case SQLITE_OK:
+			db->locked = 1;
+			break;
+		case SQLITE_BUSY:
+			sleep(1);
+			break;
+		default:
+			break;
+		}
 	}
-
-	db->locked = 1;
 }
 
 
@@ -446,7 +460,7 @@ int qdb_sqlite_store(qdbint_t db, qdb_datum key, qdb_datum val)
 
 	data = (long *) (val.data);
 
-#if HAVE_SNPRINTF
+#ifdef HAVE_SNPRINTF
 	snprintf(val1, sizeof(val1) - 1,    /* RATS: ignore (OK) */
 		 "%ld", data[0]);
 	snprintf(val2, sizeof(val2) - 1,    /* RATS: ignore (OK) */
@@ -655,15 +669,6 @@ qdb_datum qdb_sqlite_firstkey(qdbint_t db)
 }
 
 
-/*
- * Reorganise the database for better efficiency.
- */
-void qdb_sqlite_optimise(qdbint_t db)
-{
-	/* (Does nothing) */
-}
-
-
 /*
  * Return a string describing the last database error to occur.
  */
