@@ -1,7 +1,7 @@
 /*
  * Database handling prototypes, structures, and constants.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #ifndef _DATABASE_H
@@ -42,6 +42,16 @@ char *qdb_error(void);
 void qdb_unlock(qdb_t);
 void qdb_relock(qdb_t);
 
+void qdb_restore_start(qdb_t);
+void qdb_restore_end(qdb_t);
+
+/*
+ * Common library functions for backends to use.
+ */
+int qdb_int__lock(int, int, int *);
+void qdb_int__sig_block(void);
+void qdb_int__sig_unblock(void);
+
 #endif /* _DATABASE_H */
 
 /* EOF */
