@@ -1,7 +1,7 @@
 /*
  * Message handling prototypes and structures.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #ifndef _MESSAGE_H
@@ -44,7 +44,7 @@ struct msg_s {           /* structure describing an email message */
 
 msg_t msg_parse(opts_t);
 void msg_spamsubject(msg_t, char *);
-void msg_spamheader(msg_t, double);
+void msg_spamheader(msg_t, char *, double);
 void msg_spamratingheader(msg_t, double, double);
 void msg_spamlevelheader(msg_t, double, double);
 void msg_dump(msg_t);
