@@ -1,7 +1,7 @@
 /*
  * Global program option structure and the parsing function prototype.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #ifndef _OPTIONS_H
@@ -30,9 +30,11 @@ struct opts_s {           /* structure describing run-time options */
 	char *database;                /* location of the database file */
 	char *globaldb;                /* location of global database */
 	char *globaldb2;               /* location of 2nd global database */
+	char *plainmap;                /* location of plaintext map */
 	void *dbr1;                    /* first db handle, if any */
 	void *dbr2;                    /* second db handle, if any */
 	void *dbr3;                    /* third db handle, if any */
+	void *plaindata;               /* plaintext working data */
 	int db1weight;                 /* weighting multiplier for db 1 */
 	int db2weight;                 /* weighting multiplier for db 2 */
 	int db3weight;                 /* weighting multiplier for db 3 */
@@ -46,6 +48,8 @@ struct opts_s {           /* structure describing run-time options */
 	unsigned char no_filter;       /* set if we're not filtering */
 	double threshold;              /* spam threshold (default 0.9) */
 	unsigned char allowlist;       /* set if allow-list is enabled */
+	unsigned char denylist;        /* set if deny-list is enabled */
+	unsigned char modifydenylist;  /* set if acting on the deny-list */
 	unsigned int weight;           /* weighting to use when marking */
 	unsigned char noautoprune;     /* set if we've not to auto-prune */
 	unsigned char showprune;       /* show verbose prune indicator */
@@ -53,6 +57,7 @@ struct opts_s {           /* structure describing run-time options */
 	unsigned int min_token_count;  /* min tokens before giving a score */
 	unsigned long prune_max;       /* max tokens to prune at once */
 	char *subject_marker;          /* string to add to subject if spam */
+	char *header_marker;           /* string to set X-Spam header to if spam */
 	char *mergefrom;               /* database to merge data from */
 	char *emailonly;               /* email address to use in -e mode */
 	char *emailonly2;              /* extra email address for -e */
