@@ -1,7 +1,7 @@
 /*
  * Select a particular message from a mailbox.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -49,7 +49,13 @@ int mbox_select(opts_t opts, mbox_t mbox, FILE * fptr, size_t num)
 		return 1;
 	}
 
-	fread(buf, mbox->length[num], 1, fptr);
+	if (fread(buf, mbox->length[num], 1, fptr) < 1) {
+		fprintf(stderr, "%s: %s: %s\n", opts->program_name,
+			_("failed to read mailbox"), strerror(errno));
+		opts->inbuf = "";
+		opts->inbufsize = 1;
+		return 1;
+	}
 	opts->inbuf = buf;
 	opts->inbufsize = mbox->length[num];
 
