@@ -2,7 +2,7 @@
  * Main program entry point - read the command line options, then perform
  * the appropriate actions.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -70,9 +70,10 @@ int main(int argc, char **argv)
 	log_add(2, _("backends available:%s"), BACKENDS);
 
 	/*
-	 * If we're testing, marking, or tokenising, or allow-list
-	 * updating/querying (having not been given an email address), we
-	 * need to read a message from standard input before proceeding.
+	 * If we're testing, marking, or tokenising, or allow-list or
+	 * deny-list updating/querying (having not been given an email
+	 * address), we need to read a message from standard input before
+	 * proceeding.
 	 */
 	if (((opts->action == ACTION_TEST)
 	     || (opts->action == ACTION_MARK_SPAM)
@@ -111,7 +112,9 @@ int main(int argc, char **argv)
 				got =
 				    fread(buffer, 1, sizeof(buffer),
 					  stdin);
-				if (got > sizeof(buffer))
+				if ((got > 0)
+				    && ((unsigned int) got >
+					sizeof(buffer)))
 					got = sizeof(buffer);
 				if (opts->emailonly
 				    || opts->no_filter
@@ -132,8 +135,9 @@ int main(int argc, char **argv)
 					break;
 				}
 			}
+			retcode = (opts->emailonly ? 1 : 0);
 			opts_free(opts);
-			return (opts->emailonly ? 1 : 0);
+			return retcode;
 		}
 	}
 
@@ -227,11 +231,20 @@ int main(int argc, char **argv)
 		 */
 
 #ifdef P_tmpdir
-		sprintf(filename, "%.*s", (int) (sizeof(filename) - 1),
+#ifdef HAVE_SNPRINTF
+		snprintf(filename, sizeof(filename), "%.*s",
+#else
+		sprintf(filename, "%.*s",   /* RATS: ignore (OK) */
+#endif
+			(int) (sizeof(filename) - 1),
 			P_tmpdir "/qsfXXXXXX");
 #else
-		sprintf(filename, "%.*s", (int) (sizeof(filename) - 1),
-			"/tmp/qsfXXXXXX");
+#ifdef HAVE_SNPRINTF
+		snprintf(filename, sizeof(filename), "%.*s",
+#else
+		sprintf(filename, "%.*s",   /* RATS: ignore (OK) */
+#endif
+			(int) (sizeof(filename) - 1), "/tmp/qsfXXXXXX");
 #endif
 
 #ifdef HAVE_MKSTEMP
@@ -261,7 +274,12 @@ int main(int argc, char **argv)
 				       newtype);
 
 			if (newtype[0] != 0) {
-				sprintf(newfilename, "%.*s:%.*s", 32,
+#ifdef HAVE_SNPRINTF
+				snprintf(newfilename, sizeof(newfilename),
+					 "%.*s:%.*s", 32,
+#else
+				sprintf(newfilename, "%.*s:%.*s", 32,	/* RATS: ignore (OK) */
+#endif
 					newtype,
 					(int) (sizeof(filename) - 1 -
 					       strlen(newtype)), filename);
@@ -305,10 +323,19 @@ int main(int argc, char **argv)
 		 */
 
 		if (opts->globaldb) {
-			sprintf(buffer, "%.*s", (int) (sizeof(buffer) - 1),
+#ifdef HAVE_SNPRINTF
+			snprintf(buffer, sizeof(buffer), "%.*s",
+#else
+			sprintf(buffer, "%.*s",	/* RATS: ignore (OK) */
+#endif
+				(int) (sizeof(buffer) - 1),
 				opts->globaldb);
 		} else {
-			sprintf(buffer,
+#ifdef HAVE_SNPRINTF
+			snprintf(buffer, sizeof(buffer),
+#else
+			sprintf(buffer,	    /* RATS: ignore (OK) */
+#endif
 				"%.*s",
 				(int) (sizeof(buffer) - 1),
 				"/var/lib/" PACKAGE "db");
@@ -341,8 +368,8 @@ int main(int argc, char **argv)
 		if (strlen(home) /* RATS: ignore */ >(sizeof(buffer) - 64))
 			home = "/";
 
-#if HAVE_SNPRINTF
-		snprintf(buffer, sizeof(buffer) - 1,	/* RATS: ignore (OK) */
+#ifdef HAVE_SNPRINTF
+		snprintf(buffer, sizeof(buffer),	/* RATS: ignore (OK) */
 			 "%s/.%sdb", home, PACKAGE);
 #else
 		sprintf(buffer,		    /* RATS: ignore (checked above) */
@@ -380,10 +407,19 @@ int main(int argc, char **argv)
 			opts->db2weight = 10;
 
 		if (opts->globaldb2) {
-			sprintf(buffer, "%.*s", (int) (sizeof(buffer) - 1),
+#ifdef HAVE_SNPRINTF
+			snprintf(buffer, sizeof(buffer),
+#else
+			sprintf(buffer,	    /* RATS: ignore (OK) */
+#endif
+				"%.*s", (int) (sizeof(buffer) - 1),
 				opts->globaldb2);
 		} else {
-			sprintf(buffer, "%.*s",
+#ifdef HAVE_SNPRINTF
+			snprintf(buffer, sizeof(buffer), "%.*s",
+#else
+			sprintf(buffer, "%.*s",	/* RATS: ignore (OK) */
+#endif
 				(int) (sizeof(buffer) - 1),
 				"/var/lib/" PACKAGE "db2");
 		}
@@ -522,7 +558,7 @@ int main(int argc, char **argv)
 
 	if (opts->emailonly) {
 		/*
-		 * Allow-list manipulation / querying mode.
+		 * Allow-list and deny-list manipulation / querying mode.
 		 */
 		if (strcmp(opts->emailonly, "MSG") == 0) {
 			if ((msg) && (msg->sender))
@@ -530,7 +566,11 @@ int main(int argc, char **argv)
 			if ((msg) && (msg->envsender))
 				opts->emailonly2 = msg->envsender;
 		}
-		retcode = spam_allowlist_manage(opts);
+		if (opts->denylist) {
+			retcode = spam_denylist_manage(opts);
+		} else {
+			retcode = spam_allowlist_manage(opts);
+		}
 		log_errdump(opts->program_name);
 		opts_free(opts);
 		qdb_close(dbr1);
@@ -638,7 +678,7 @@ int main(int argc, char **argv)
 		log_add(3, _("images found: %d"), msg->num_images);
 
 		if (!opts->no_header)
-			msg_spamheader(msg, score);
+			msg_spamheader(msg, opts->header_marker, score);
 		if (opts->add_rating) {
 			msg_spamratingheader(msg, score, opts->threshold);
 			if (opts->no_filter) {
