@@ -36,7 +36,6 @@ extern int minioptind, miniopterr, minioptopt;
 
 
 void display_help(void);
-void display_license(void);
 void display_version(void);
 
 
@@ -57,7 +56,7 @@ void opts_free(opts_t opts)
 
 /*
  * Parse the given command-line arguments into an opts_t object, handling
- * "help", "license" and "version" options internally.
+ * "help" and "version" options internally.
  *
  * Returns an opts_t, or 0 on error.
  *
@@ -70,7 +69,6 @@ opts_t opts_parse(int argc, char **argv)
 #ifdef HAVE_GETOPT_LONG
 	struct option long_options[] = {
 		{"help", 0, 0, 'h'},
-		{"license", 0, 0, 'l'},
 		{"version", 0, 0, 'V'},
 		{"subject", 0, 0, 's'},
 		{"subject-marker", 1, 0, 'S'},
@@ -127,8 +125,7 @@ opts_t opts_parse(int argc, char **argv)
 	};
 	int option_index = 0;
 #endif
-	char *short_options =
-	    "hlVsS:H:nrAtoayL:Q:e:mMd:g:P:w:NTX:pDROBE:v";
+	char *short_options = "hVsS:H:nrAtoayL:Q:e:mMd:g:P:w:NTX:pDROBE:v";
 	int c;
 	opts_t opts;
 
@@ -184,11 +181,6 @@ opts_t opts_parse(int argc, char **argv)
 			opts->action = ACTION_NONE;
 			return opts;
 			break;
-		case 'l':
-			display_license();
-			opts->action = ACTION_NONE;
-			return opts;
-			break;
 		case 'V':
 			display_version();
 			opts->action = ACTION_NONE;
