@@ -13,7 +13,7 @@
  */
 void display_version(void)
 {
-	printf( /* RATS: ignore */ _("%s %s - Copyright(C) %s %s"),
+	printf( /* RATS: ignore */ _("%s %s - Copyright (C) %s %s"),
 	       PROGRAM_NAME, VERSION, COPYRIGHT_YEAR, COPYRIGHT_HOLDER);
 	printf("\n%s%s", _("Backends available:"), BACKENDS);
 
@@ -22,24 +22,13 @@ void display_version(void)
 	printf("\n\n");
 	printf("%s",
 	       _("This program is free software, and is being distributed "
-		 "under the\nterms of the Artistic License."));
+		 "under the\nterms of the Artistic License 2.0."));
 	printf("\n\n");
 	printf("%s", _
 	       ("This program is distributed in the hope that it will be useful,\n"
 		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
 		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."));
 	printf("\n\n");
-#ifdef HAVE_GETOPT_LONG
-	printf(				    /* RATS: ignore */
-		      _
-		      ("For more information, please run `%s --license'."),
-		      PROGRAM_NAME);
-#else
-	printf(				    /* RATS: ignore */
-		      _("For more information, please run `%s -l'."),
-		      PROGRAM_NAME);
-#endif
-	printf("\n");
 }
 
 /* EOF */
