@@ -1,7 +1,7 @@
 /*
  * Dump a mail message to stdout.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -39,13 +39,20 @@ void msg_dump(msg_t msg)
 	for (i = 0; i < msg->num_headers; i++) {
 		if (msg->header[i] == NULL)
 			continue;
-		fwrite(msg->header[i], strlen(msg->header[i]), 1, stdout);
-		fwrite("\n", 1, 1, stdout);
+		sent =
+		    fwrite(msg->header[i], strlen(msg->header[i]), 1,
+			   stdout);
+		if (sent <= 0)
+			break;
+		sent = fwrite("\n", 1, 1, stdout);
+		if (sent <= 0)
+			break;
 	}
 
 	log_dump("X-QSF-Info: ");
 
-	fwrite("\n", 1, 1, stdout);
+	if (fwrite("\n", 1, 1, stdout) <= 0)
+		return;
 
 	pos = 0;
 	while (pos < msg->body_size) {
