@@ -1,7 +1,7 @@
 /*
  * Functions for parsing and handling mail messages.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -210,7 +210,7 @@ static int msg_parse__header(opts_t opts, msg_t msg)
 					strerror(errno));
 				return 1;
 			}
-			strcpy(newptr, "\n--");
+			memcpy(newptr, "\n--", 3);
 			strncpy(newptr + 3,
 				msg->original + boundstart,
 				msg->_pos - boundstart);
@@ -399,11 +399,17 @@ static int msg_parse__content(opts_t opts, msg_t msg)
 		MD5Init(&md5c);
 		MD5Update(&md5c, (unsigned char *) content, partsize);
 		MD5Final(digest, &md5c);
-		strcpy(digeststr, " z");
+		memcpy(digeststr, " z", 2);
 		for (n = 0; n < 16; n++) {
-			sprintf(digeststr + 2 + 2 * n, "%02X", digest[n]);
+#ifdef HAVE_SNPRINTF
+			snprintf(digeststr + 2 + 2 * n,
+				 sizeof(digeststr) - 2 - 2 * n,
+#else
+			sprintf(digeststr + 2 + 2 * n,	/* RATS: ignore */
+#endif
+				"%02X", digest[n]);
 		}
-		strcat(digeststr, "z ");
+		memcpy(digeststr + strlen(digeststr), "z \000", 3);
 		if (msg_addcontent
 		    (opts, msg, digeststr, strlen(digeststr)))
 			return 1;
