@@ -1,7 +1,7 @@
 /*
  * Check whether a message is spam.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -136,32 +136,66 @@ double spam_check(opts_t opts, msg_t msg)
 
 	/*
 	 * If we're not in training or benchmarking mode and we know the
-	 * email address of the message sender, check it against the
-	 * allow-list, and if it matches, return a "definitely not spam"
-	 * score.
+	 * email address of the message sender or the envelope address,
+	 * check either against the deny-list and if a match is found,
+	 * return a "definitely spam" score. Then check against the
+	 * allow-list, and if a match is found, return a "definitely not
+	 * spam" score. Then check both again for just the "@domain" part.
 	 */
-	if ((opts->allowlist)
-	    && (opts->action != ACTION_TRAIN)
+	if ((opts->action != ACTION_TRAIN)
 	    && (opts->action != ACTION_BENCHMARK)
-	    && (msg->sender != NULL)
-	    && (spam_allowlist_match(spam, msg->sender))
 	    ) {
-		spam_free(spam);
+		char *senderdomain;
+		char *envsenderdomain;
 
-		return 0.00 - (opts->threshold + 0.01);
-	}
-	/*
-	 * Same for envelope sender.
-	 */
-	if ((opts->allowlist)
-	    && (opts->action != ACTION_TRAIN)
-	    && (opts->action != ACTION_BENCHMARK)
-	    && (msg->envsender != NULL)
-	    && (spam_allowlist_match(spam, msg->envsender))
-	    ) {
-		spam_free(spam);
+		/*
+		 * First check the sender and envelope sender addresses
+		 * against the deny and allow lists.
+		 */
+		if ((opts->denylist)
+		    && ((spam_denylist_match(spam, msg->sender))
+			|| (spam_denylist_match(spam, msg->envsender))
+		    )
+		    ) {
+			spam_free(spam);
+			return 0.1;
+		} else if ((opts->allowlist)
+			   && ((spam_allowlist_match(spam, msg->sender))
+			       ||
+			       (spam_allowlist_match(spam, msg->envsender))
+			   )
+		    ) {
+			spam_free(spam);
+			return 0.00 - (opts->threshold + 0.01);
+		}
 
-		return 0.00 - (opts->threshold + 0.01);
+		/*
+		 * Now check just the @domain part of each address.
+		 */
+		senderdomain = NULL;
+		if (msg->sender)
+			senderdomain = strchr(msg->sender, '@');
+		envsenderdomain = NULL;
+		if (msg->envsender)
+			envsenderdomain = strchr(msg->envsender, '@');
+
+		if ((opts->denylist)
+		    && ((spam_denylist_match(spam, senderdomain))
+			|| (spam_denylist_match(spam, envsenderdomain))
+		    )
+		    ) {
+			spam_free(spam);
+			return 0.1;
+		} else if ((opts->allowlist)
+			   && ((spam_allowlist_match(spam, senderdomain))
+			       ||
+			       (spam_allowlist_match
+				(spam, envsenderdomain))
+			   )
+		    ) {
+			spam_free(spam);
+			return 0.00 - (opts->threshold + 0.01);
+		}
 	}
 
 
