@@ -1,7 +1,7 @@
 /*
  * Token checksumming functions.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -28,10 +28,15 @@ unsigned char *spam_checksum(char *key, int len)
 	MD5Update(&md5c, (unsigned char *) key, len);
 	MD5Final(digest, &md5c);
 
-	strcpy(resultstr, "!");
+	memcpy(resultstr, "!\000", 2);
 	for (i = 0; i < 16; i++) {
+#ifdef HAVE_SNPRINTF
+		snprintf(buf, sizeof(buf), "%02x", digest[i]);
+#else
 		sprintf(buf, "%02x", digest[i]);
-		strcat(resultstr, buf);	    /* RATS: ignore (len OK) */
+#endif
+		buf[2] = 0;
+		memcpy(resultstr + strlen(resultstr), buf, 3);
 	}
 
 	ptr = (unsigned char *) (strdup(resultstr));
