@@ -1,7 +1,7 @@
 /*
  * Database fetch/store functions.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -48,6 +48,7 @@ void spam_fetch(spam_t spam, char *key, int len, long *val1, long *val2,
 	if ((key[0] != ' ')
 	    && (key[0] != '!')
 	    && (key[0] != '?')
+	    && (key[0] != '\\')
 	    && (key[0] != '.')
 	    ) {
 		needcksum = 1;
@@ -117,6 +118,7 @@ void spam_store(opts_t opts, char *key, int len, long val1, long val2,
 	if ((key[0] != ' ')
 	    && (key[0] != '!')
 	    && (key[0] != '?')
+	    && (key[0] != '\\')
 	    && (key[0] != '.')
 	    ) {
 		needcksum = 1;
@@ -131,8 +133,13 @@ void spam_store(opts_t opts, char *key, int len, long val1, long val2,
 	dat[2] = val3;
 	qdb_store(opts->dbw, dkey, dval);
 
-	if (needcksum)
+	if (needcksum) {
+		if (opts->plainmap) {
+			spam_plaintext_update(opts, (char *) (dkey.data),
+					      dkey.size, key, len);
+		}
 		free(dkey.data);
+	}
 }
 
 
