@@ -1,7 +1,7 @@
 /*
  * Functions for recognising mail as spam and updating the database.
  *
- * Copyright 2005 Andrew Wood, distributed under the Artistic License.
+ * Copyright 2007 Andrew Wood, distributed under the Artistic License.
  */
 
 #include "config.h"
@@ -25,7 +25,15 @@ int spam_update(opts_t opts, msg_t msg, int type)
 		return 1;
 	}
 
-	if (opts->allowlist) {
+	if (opts->denylist && opts->modifydenylist) {
+		if (type == SPAM) {
+			spam_denylist_add(opts, msg->sender);
+			spam_denylist_add(opts, msg->envsender);
+		} else {
+			spam_denylist_remove(opts, msg->sender);
+			spam_denylist_remove(opts, msg->envsender);
+		}
+	} else if (opts->allowlist) {
 		if (type == SPAM) {
 			spam_allowlist_remove(opts, msg->sender);
 			spam_allowlist_remove(opts, msg->envsender);
